"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const help_1 = __importDefault(require("./help"));
const picocolors_1 = require("picocolors");
const lodash_1 = require("lodash");
const child_process_1 = require("child_process");
const node_util_1 = __importDefault(require("node:util"));
class OdhinPlaywrightReporter {
    constructor(execOptions) {
        // Const Variables
        this.OUTPUT_FOLDER = "playwright-html-report";
        this.TITLE = "Playwright HTML Report";
        this.TEST_FOLDER = "tests";
        this.INDEX_FILE = "index.html";
        this.execOptions = execOptions;
        this.totalDuration = 0;
        this.help = new help_1.default(this.execOptions);
        this.testCases = [];
        this.execInfo = {
            browserName: "",
            totalPassed: 0,
            totalFailed: 0,
            totalTimedOut: 0,
            totalSkipped: 0,
            totalInterrupted: 0,
            totalTests: 0,
            startTime: "",
            endTime: "",
            execFiles: [],
            execBrowsers: [],
        };
        this.execOptions.testFolder =
            this.execOptions.testFolder !== undefined ? this.execOptions.testFolder : this.TEST_FOLDER;
        this.execOptions.title = this.execOptions.title !== undefined ? this.execOptions.title : this.TITLE;
        this.execOptions.testEnvironment =
            this.execOptions.testEnvironment !== undefined ? this.execOptions.testEnvironment : "";
        this.execOptions.project = this.execOptions.project !== undefined ? this.execOptions.project : "";
        this.execOptions.calcVersion = this.execOptions.calcVersion !== undefined ? this.execOptions.calcVersion : "";
        this.execOptions.embedAssets = this.execOptions.embedAssets !== undefined ? this.execOptions.embedAssets : true;
        this.execOptions.embedAttachments =
            this.execOptions.embedAttachments !== undefined ? this.execOptions.embedAttachments : true;
        this.execOptions.outputFolder =
            this.execOptions.outputFolder !== undefined ? this.execOptions.outputFolder : this.OUTPUT_FOLDER;
        this.execOptions.minifyAssets = this.execOptions.minifyAssets !== undefined ? this.execOptions.minifyAssets : true;
        this.execOptions.startServer = this.execOptions.startServer !== undefined ? this.execOptions.startServer : true;
        this.execOptions.consoleLog = this.execOptions.consoleLog !== undefined ? this.execOptions.consoleLog : true;
        this.execOptions.consoleError = this.execOptions.consoleError !== undefined ? this.execOptions.consoleError : true;
        this.help.showError = this.execOptions.consoleError;
        // Remove output folder if exists
        fs_1.default.rmSync(this.execOptions.outputFolder, { recursive: true, force: true });
        try {
            // Load HTML Files
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/base.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.base = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/runInfo.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.runInfo = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/chart.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.chart = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/chartInfo.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.chartInfo = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/chartStatus.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.chartStatus = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/chartFile.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.chartFile = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/chartBrowser.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.chartBrowser = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/chartJs.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.chartJs = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/content.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.content = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/summary.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.summary = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/summaryRow.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.summaryRow = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/testCase.html"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.testCaseTemplate = data;
            });
            // Load CSS Files
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/css/style.css"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.css = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/css/jquery.dataTables.css"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.jqueryDataTablesCss = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/css/bootstrap.css"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.bootstrapCss = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/css/prism.css"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.prismCss = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/css/img-comparison-slider.css"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.imgComparisonSliderCss = data;
            });
            // Load JS Files
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/jquery-3.5.1.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.jQuery = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/script.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.js = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/chart.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.chartJsLibrary = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/jquery.dataTables.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.jqueryDataTablesJs = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/dataTables.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.dataTables = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/bootstrap.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.bootstrapJs = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/prism.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.prismJs = data;
            });
            fs_1.default.readFile(path_1.default.resolve(__dirname, "html/assets/js/img-comparison-slider.js"), "utf-8", (err, data) => {
                if (err)
                    throw err;
                this.imgComparisonSliderJs = data;
            });
        }
        catch (error) {
            if (this.execOptions.consoleError === true) {
                console.log(error);
            }
        }
        if (this.execOptions.consoleLog === true) {
            console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(`-------------------------------------`))}`);
            console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(` `))}`);
        }
    }
    onBegin(config, suite) {
        this.execInfo.startTime = new Date().toUTCString();
        config.projects.forEach((p, index) => {
            this.execInfo.browserName += index > 0 ? ", " : "";
            this.execInfo.browserName += p.name.charAt(0).toUpperCase() + p.name.slice(1);
        });
        if (this.execOptions.consoleLog === true) {
            console.log(`${(0, picocolors_1.bold)((0, picocolors_1.yellow)(`⏺  `))}${(0, picocolors_1.bold)((0, picocolors_1.blue)(`Starting the run with ${suite.allTests().length} tests`))}`);
            console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(` `))}`);
        }
    }
    /*
      onTestBegin(test: TestCase) {
          console.log(`Starting test ${test.title}`);
      }
   */
    onTestEnd(testCase, result) {
        this.execInfo.endTime = new Date().toUTCString();
        this.testCases.push({
            testCase: testCase,
            result: result,
        });
        this.totalDuration = this.totalDuration + result.duration;
        let browser = "";
        let tst = testCase;
        if (tst._projectId !== undefined) {
            browser = tst._projectId;
            browser = browser.charAt(0).toUpperCase() + browser.slice(1) + " | ";
        }
        if (result.status.toString() === "passed" || result.retry === testCase.parent["_entries"][0].retries) {
            // Update Files
            let fileUpdate = false;
            this.execInfo.execFiles.forEach((f, index) => {
                if (f.filename === this.help.getShortFilePath(testCase["_requireFile"], this.execOptions.testFolder)) {
                    this.execInfo.execFiles[index].totalTests++;
                    this.execInfo.execFiles[index].totalDuration = this.execInfo.execFiles[index].totalDuration + result.duration;
                    (this.execInfo.execFiles[index].totalPassed =
                        result.status.toString() == "passed"
                            ? this.execInfo.execFiles[index].totalPassed + 1
                            : this.execInfo.execFiles[index].totalPassed),
                        (this.execInfo.execFiles[index].totalFailed =
                            result.status.toString() == "failed"
                                ? this.execInfo.execFiles[index].totalFailed + 1
                                : this.execInfo.execFiles[index].totalFailed),
                        (this.execInfo.execFiles[index].totalTimedOut =
                            result.status.toString() == "timedOut"
                                ? this.execInfo.execFiles[index].totalTimedOut + 1
                                : this.execInfo.execFiles[index].totalTimedOut),
                        (this.execInfo.execFiles[index].totalSkipped =
                            result.status.toString() == "skipped"
                                ? this.execInfo.execFiles[index].totalSkipped + 1
                                : this.execInfo.execFiles[index].totalSkipped),
                        (this.execInfo.execFiles[index].totalInterrupted =
                            result.status.toString() == "interrupted"
                                ? this.execInfo.execFiles[index].totalInterrupted + 1
                                : this.execInfo.execFiles[index].totalInterrupted),
                        (this.execInfo.execFiles[index].endTime = new Date().toUTCString());
                    fileUpdate = true;
                }
            });
            if (!fileUpdate) {
                this.execInfo.execFiles.push({
                    filename: this.help.getShortFilePath(testCase["_requireFile"], this.execOptions.testFolder),
                    color: this.help.getRandomColor(),
                    totalPassed: result.status.toString() == "passed" ? 1 : 0,
                    totalFailed: result.status.toString() == "failed" ? 1 : 0,
                    totalTimedOut: result.status.toString() == "timedOut" ? 1 : 0,
                    totalSkipped: result.status.toString() == "skipped" ? 1 : 0,
                    totalInterrupted: result.status.toString() == "interrupted" ? 1 : 0,
                    totalTests: 1,
                    totalDuration: result.duration,
                    startTime: new Date(new Date().getTime() - result.duration).toUTCString(),
                    endTime: new Date().toUTCString(),
                });
            }
            // Update Browsers
            let browserUpdate = false;
            this.execInfo.execBrowsers.forEach((b, index) => {
                if (b.browser === testCase["_projectId"]) {
                    this.execInfo.execBrowsers[index].totalTests++;
                    this.execInfo.execBrowsers[index].totalDuration =
                        this.execInfo.execBrowsers[index].totalDuration + result.duration;
                    (this.execInfo.execBrowsers[index].totalPassed =
                        result.status.toString() == "passed"
                            ? this.execInfo.execBrowsers[index].totalPassed + 1
                            : this.execInfo.execBrowsers[index].totalPassed),
                        (this.execInfo.execBrowsers[index].totalFailed =
                            result.status.toString() == "failed"
                                ? this.execInfo.execBrowsers[index].totalFailed + 1
                                : this.execInfo.execBrowsers[index].totalFailed),
                        (this.execInfo.execBrowsers[index].totalTimedOut =
                            result.status.toString() == "timedOut"
                                ? this.execInfo.execBrowsers[index].totalTimedOut + 1
                                : this.execInfo.execBrowsers[index].totalTimedOut),
                        (this.execInfo.execBrowsers[index].totalSkipped =
                            result.status.toString() == "skipped"
                                ? this.execInfo.execBrowsers[index].totalSkipped + 1
                                : this.execInfo.execBrowsers[index].totalSkipped),
                        (this.execInfo.execBrowsers[index].totalInterrupted =
                            result.status.toString() == "interrupted"
                                ? this.execInfo.execBrowsers[index].totalInterrupted + 1
                                : this.execInfo.execBrowsers[index].totalInterrupted),
                        (this.execInfo.execBrowsers[index].endTime = new Date().toUTCString());
                    browserUpdate = true;
                }
            });
            if (!browserUpdate && testCase.parent.parent !== undefined) {
                this.execInfo.execBrowsers.push({
                    browser: testCase["_projectId"],
                    color: this.help.getRandomColor(),
                    totalPassed: result.status.toString() == "passed" ? 1 : 0,
                    totalFailed: result.status.toString() == "failed" ? 1 : 0,
                    totalTimedOut: result.status.toString() == "timedOut" ? 1 : 0,
                    totalSkipped: result.status.toString() == "skipped" ? 1 : 0,
                    totalInterrupted: result.status.toString() == "interrupted" ? 1 : 0,
                    totalTests: 1,
                    totalDuration: result.duration,
                    startTime: new Date(new Date().getTime() - result.duration).toUTCString(),
                    endTime: new Date().toUTCString(),
                });
            }
            switch (result.status.toString()) {
                case "passed":
                    this.execInfo.totalPassed++;
                    if (this.execOptions.consoleLog === true) {
                        console.log(`${(0, picocolors_1.bold)((0, picocolors_1.green)(`✅ ${browser}${testCase.title}`))}`);
                    }
                    break;
                case "failed":
                    this.execInfo.totalFailed++;
                    if (this.execOptions.consoleLog === true) {
                        console.log(`${(0, picocolors_1.bold)((0, picocolors_1.red)(`⛔ ${browser}${testCase.title}`))}`);
                    }
                    break;
                case "timedOut":
                    this.execInfo.totalTimedOut++;
                    if (this.execOptions.consoleLog === true) {
                        console.log(`${(0, picocolors_1.bold)((0, picocolors_1.magenta)(`⛔ ${browser}${testCase.title}`))}`);
                    }
                    break;
                case "skipped":
                    this.execInfo.totalSkipped++;
                    if (this.execOptions.consoleLog === true) {
                        console.log(`${(0, picocolors_1.bold)((0, picocolors_1.yellow)(`🚫 ${browser}${testCase.title}`))}`);
                    }
                    break;
                case "interrupted":
                    this.execInfo.totalInterrupted++;
                    if (this.execOptions.consoleLog === true) {
                        console.log(`${(0, picocolors_1.bold)((0, picocolors_1.gray)(`🚫 ${browser}${testCase.title}`))}`);
                    }
                    break;
            }
            this.execInfo.totalTests++;
        }
    }
    async onEnd() {
        // Edit Chart Status
        this.chartStatus = this.chartStatus.replace("{{.ChartId}}", "chart-status");
        this.chartStatus = this.chartStatus.replace("{{.TotalTests}}", this.execInfo.totalTests.toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestPassedValue}}", this.execInfo.totalPassed.toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestFailedValue}}", this.execInfo.totalFailed.toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestTimedOutValue}}", this.execInfo.totalTimedOut.toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestSkippedValue}}", this.execInfo.totalSkipped.toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestInterruptedValue}}", this.execInfo.totalInterrupted.toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestPassedPercentage}}", (0, lodash_1.round)((this.execInfo.totalPassed / this.execInfo.totalTests) * 100, 2).toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestFailedPercentage}}", (0, lodash_1.round)((this.execInfo.totalFailed / this.execInfo.totalTests) * 100, 2).toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestTimedOutPercentage}}", (0, lodash_1.round)((this.execInfo.totalTimedOut / this.execInfo.totalTests) * 100, 2).toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestSkippedPercentage}}", (0, lodash_1.round)((this.execInfo.totalSkipped / this.execInfo.totalTests) * 100, 2).toString());
        this.chartStatus = this.chartStatus.replaceAll("{{.TestInterruptedPercentage}}", (0, lodash_1.round)((this.execInfo.totalInterrupted / this.execInfo.totalTests) * 100, 2).toString());
        this.chartJs = this.chartJs.replaceAll("{{.PassedStatusChart}}", this.execInfo.totalPassed.toString());
        this.chartJs = this.chartJs.replaceAll("{{.FailedStatusChart}}", this.execInfo.totalFailed.toString());
        this.chartJs = this.chartJs.replaceAll("{{.TimedOutStatusChart}}", this.execInfo.totalTimedOut.toString());
        this.chartJs = this.chartJs.replaceAll("{{.SkippedStatusChart}}", this.execInfo.totalSkipped.toString());
        this.chartJs = this.chartJs.replaceAll("{{.InterruptedStatusChart}}", this.execInfo.totalInterrupted.toString());
        // Edit Chart File
        let chartFileDetail = this.chart;
        let chartJsFileLabels = "";
        let chartJsFileData = "";
        let chartJsFileDataColor = "";
        let chartFileRows = "";
        chartFileDetail = chartFileDetail.replace("{{.ChartId}}", "chart-file");
        chartFileDetail = chartFileDetail.replace("{{.TotalTests}}", this.execInfo.totalTests.toString());
        chartFileDetail = chartFileDetail.replace("{{.RowSpan}}", this.execInfo.execFiles.length.toString());
        this.execInfo.execFiles.forEach((file, index) => {
            let chartRow = this.chartInfo;
            if (index == 0) {
                chartRow = chartRow.replace("{{.Label}}", file.filename);
                chartRow = chartRow.replace("{{.Value}}", file.totalTests.toString());
                chartRow = chartRow.replace("{{.Percentage}}", (0, lodash_1.round)((file.totalTests / this.execInfo.totalTests) * 100, 2).toString());
                chartJsFileLabels += "'" + file.filename + " (" + file.totalTests.toString() + ")" + "'";
                chartJsFileData += file.totalTests.toString();
                chartJsFileDataColor += "'" + file.color + "'";
            }
            else {
                chartRow = chartRow.replace("{{.Label}}", file.filename);
                chartRow = chartRow.replace("{{.Value}}", file.totalTests.toString());
                chartRow = chartRow.replace("{{.Percentage}}", (0, lodash_1.round)((file.totalTests / this.execInfo.totalTests) * 100, 2).toString());
                chartJsFileLabels += ",'" + file.filename + " (" + file.totalTests.toString() + ")" + "'";
                chartJsFileData += "," + file.totalTests.toString();
                chartJsFileDataColor += ",'" + file.color + "'";
            }
            chartRow = chartRow.replaceAll("{{.Color}}", file.color);
            chartFileRows += chartRow;
        });
        this.chartFile = this.chartFile.replace("{{.Chart}}", chartFileDetail);
        this.chartFile = this.chartFile.replaceAll("{{.ChartFile}}", chartFileRows);
        this.chartJs = this.chartJs.replaceAll("{{.FilesChartLabels}}", chartJsFileLabels);
        this.chartJs = this.chartJs.replaceAll("{{.FilesChartData}}", chartJsFileData);
        this.chartJs = this.chartJs.replaceAll("{{.FilesChartDataColor}}", chartJsFileDataColor);
        // Edit Chart Browser
        let chartBrowserDetail = this.chart;
        let chartJsBrowserLabels = "";
        let chartJsBrowserData = "";
        let chartJsBrowserDataColor = "";
        let chartBrowserRows = "";
        chartBrowserDetail = chartBrowserDetail.replace("{{.ChartId}}", "chart-browser");
        chartBrowserDetail = chartBrowserDetail.replace("{{.TotalTests}}", this.execInfo.totalTests.toString());
        chartBrowserDetail = chartBrowserDetail.replace("{{.RowSpan}}", this.execInfo.execFiles.length.toString());
        this.execInfo.execBrowsers.forEach((browser, index) => {
            let chartRow = this.chartInfo;
            if (index == 0) {
                chartRow = chartRow.replace("{{.Label}}", browser.browser);
                chartRow = chartRow.replace("{{.Value}}", browser.totalTests.toString());
                chartRow = chartRow.replace("{{.Percentage}}", (0, lodash_1.round)((browser.totalTests / this.execInfo.totalTests) * 100, 2).toString());
                chartJsBrowserLabels += "'" + browser.browser + " (" + browser.totalTests.toString() + ")" + "'";
                chartJsBrowserData += browser.totalTests.toString();
                chartJsBrowserDataColor += "'" + browser.color + "'";
            }
            else {
                chartRow = chartRow.replace("{{.Label}}", browser.browser);
                chartRow = chartRow.replace("{{.Value}}", browser.totalTests.toString());
                chartRow = chartRow.replace("{{.Percentage}}", (0, lodash_1.round)((browser.totalTests / this.execInfo.totalTests) * 100, 2).toString());
                chartJsBrowserLabels += ",'" + browser.browser + " (" + browser.totalTests.toString() + ")" + "'";
                chartJsBrowserData += "," + browser.totalTests.toString();
                chartJsBrowserDataColor += ",'" + browser.color + "'";
            }
            chartRow = chartRow.replaceAll("{{.Color}}", browser.color);
            chartBrowserRows += chartRow;
        });
        this.chartBrowser = this.chartBrowser.replace("{{.Chart}}", chartBrowserDetail);
        this.chartBrowser = this.chartBrowser.replaceAll("{{.ChartBrowser}}", chartBrowserRows);
        this.chartJs = this.chartJs.replaceAll("{{.BrowsersChartLabels}}", chartJsBrowserLabels);
        this.chartJs = this.chartJs.replaceAll("{{.BrowsersChartData}}", chartJsBrowserData);
        this.chartJs = this.chartJs.replaceAll("{{.BrowsersChartDataColor}}", chartJsBrowserDataColor);
        // Edit Summary Files
        let summaryFiles = this.summary;
        summaryFiles = summaryFiles.replace("{{.SummaryTitleName}}", "Status by test file");
        summaryFiles = summaryFiles.replace("{{.SummaryColumnName}}", "Filename");
        let filesSummaryRows = "";
        this.execInfo.execFiles.forEach((file) => {
            let filesSummaryRow = this.summaryRow;
            filesSummaryRow = filesSummaryRow.replace("{{.Name}}", file.filename);
            filesSummaryRow = filesSummaryRow.replace("{{.TotalTests}}", file.totalTests.toString());
            filesSummaryRow = filesSummaryRow.replace("{{.TotalExecutionTime}}", this.help.convertMsToTime(new Date(file.endTime).getTime() - new Date(file.startTime).getTime()));
            filesSummaryRow = filesSummaryRow.replace("{{.PassedValue}}", file.totalPassed.toString());
            filesSummaryRow = filesSummaryRow.replace("{{.PassedPercentage}}", (0, lodash_1.round)((file.totalPassed / file.totalTests) * 100, 2).toString());
            filesSummaryRow = filesSummaryRow.replace("{{.FailedValue}}", file.totalFailed.toString());
            filesSummaryRow = filesSummaryRow.replace("{{.FailedPercentage}}", (0, lodash_1.round)((file.totalFailed / file.totalTests) * 100, 2).toString());
            filesSummaryRow = filesSummaryRow.replace("{{.TimedOutValue}}", file.totalTimedOut.toString());
            filesSummaryRow = filesSummaryRow.replace("{{.TimedOutPercentage}}", (0, lodash_1.round)((file.totalTimedOut / file.totalTests) * 100, 2).toString());
            filesSummaryRow = filesSummaryRow.replace("{{.SkippedValue}}", file.totalSkipped.toString());
            filesSummaryRow = filesSummaryRow.replace("{{.SkippedPercentage}}", (0, lodash_1.round)((file.totalSkipped / file.totalTests) * 100, 2).toString());
            filesSummaryRow = filesSummaryRow.replace("{{.InterruptedValue}}", file.totalInterrupted.toString());
            filesSummaryRow = filesSummaryRow.replace("{{.InterruptedPercentage}}", (0, lodash_1.round)((file.totalInterrupted / file.totalTests) * 100, 2).toString());
            filesSummaryRows += filesSummaryRow;
        });
        summaryFiles = summaryFiles.replace("{{.Data}}", filesSummaryRows);
        // Edit Summary Browsers
        let summaryBrowsers = this.summary;
        summaryBrowsers = summaryBrowsers.replace("{{.SummaryTitleName}}", "Status by browser");
        summaryBrowsers = summaryBrowsers.replace("{{.SummaryColumnName}}", "Browser");
        let browsersSummaryRows = "";
        this.execInfo.execBrowsers.forEach((browser) => {
            let browserSummaryRow = this.summaryRow;
            browserSummaryRow = browserSummaryRow.replace("{{.Name}}", browser.browser);
            browserSummaryRow = browserSummaryRow.replace("{{.TotalTests}}", browser.totalTests.toString());
            browserSummaryRow = browserSummaryRow.replace("{{.TotalExecutionTime}}", this.help.convertMsToTime(new Date(browser.endTime).getTime() - new Date(browser.startTime).getTime()));
            browserSummaryRow = browserSummaryRow.replace("{{.PassedValue}}", browser.totalPassed.toString());
            browserSummaryRow = browserSummaryRow.replace("{{.PassedPercentage}}", (0, lodash_1.round)((browser.totalPassed / browser.totalTests) * 100, 2).toString());
            browserSummaryRow = browserSummaryRow.replace("{{.FailedValue}}", browser.totalFailed.toString());
            browserSummaryRow = browserSummaryRow.replace("{{.FailedPercentage}}", (0, lodash_1.round)((browser.totalFailed / browser.totalTests) * 100, 2).toString());
            browserSummaryRow = browserSummaryRow.replace("{{.TimedOutValue}}", browser.totalTimedOut.toString());
            browserSummaryRow = browserSummaryRow.replace("{{.TimedOutPercentage}}", (0, lodash_1.round)((browser.totalTimedOut / browser.totalTests) * 100, 2).toString());
            browserSummaryRow = browserSummaryRow.replace("{{.SkippedValue}}", browser.totalSkipped.toString());
            browserSummaryRow = browserSummaryRow.replace("{{.SkippedPercentage}}", (0, lodash_1.round)((browser.totalSkipped / browser.totalTests) * 100, 2).toString());
            browserSummaryRow = browserSummaryRow.replace("{{.InterruptedValue}}", browser.totalInterrupted.toString());
            browserSummaryRow = browserSummaryRow.replace("{{.InterruptedPercentage}}", (0, lodash_1.round)((browser.totalInterrupted / browser.totalTests) * 100, 2).toString());
            browsersSummaryRows += browserSummaryRow;
        });
        summaryBrowsers = summaryBrowsers.replace("{{.Data}}", browsersSummaryRows);
        // Edit RunInfo
        this.runInfo = this.runInfo.replaceAll("{{.Project}}", this.execOptions.project);
        this.runInfo = this.runInfo.replaceAll("{{.CalcEngineVersion}}", this.execOptions.calcVersion);
        this.runInfo = this.runInfo.replaceAll("{{.TestEnvironment}}", this.execOptions.testEnvironment);
        this.runInfo = this.runInfo.replaceAll("{{.StartTime}}", this.execInfo.startTime);
        this.runInfo = this.runInfo.replaceAll("{{.EndTime}}", this.execInfo.endTime);
        this.runInfo = this.runInfo.replaceAll("{{.TotalExecutionTime}}", this.help.convertMsToTime(new Date(this.execInfo.endTime).getTime() - new Date(this.execInfo.startTime).getTime()));
        // Prepara test data
        let testCases = "";
        let showRetries = false;
        for (const tc of this.testCases) {
            if (tc.testCase.retries > 0) {
                showRetries = true;
                break;
            }
        }
        this.testCases.forEach((testCase) => {
            let testCaseId = testCase.testCase.id + "-" + testCase.result.retry;
            let testCaseHtml = this.testCaseTemplate;
            testCaseHtml = testCaseHtml.replaceAll("{{.ModalID}}", testCaseId);
            if (showRetries === true) {
                testCaseHtml = testCaseHtml.replaceAll("{{.Retry}}", '<td class="{{.TDClassesCss}}">' +
                    (testCase.result.retry > 0 ? testCase.result.retry.toString() : "") +
                    "</td>");
            }
            else {
                testCaseHtml = testCaseHtml.replaceAll("{{.Retry}}", "");
            }
            testCaseHtml = testCaseHtml.replaceAll("{{.TRClassesCss}}", "result-status-" + testCase.result.status);
            testCaseHtml = testCaseHtml.replaceAll("{{.TDClassesCss}}", "result-status-" + testCase.result.status);
            testCaseHtml = testCaseHtml.replaceAll("{{.ModalHeaderClassesCss}}", "label-status-" + testCase.result.status);
            testCaseHtml = testCaseHtml.replaceAll("{{.ModalHeaderStatus}}", testCase.result.status);
            testCaseHtml = testCaseHtml.replaceAll("{{.Title}}", testCase.testCase.title);
            testCaseHtml = testCaseHtml.replaceAll("{{.Status}}", testCase.result.status);
            testCaseHtml = testCaseHtml.replaceAll("{{.File}}", this.help.getShortFilePath(testCase.testCase["_requireFile"], this.execOptions.testFolder));
            testCaseHtml = testCaseHtml.replaceAll("{{.Browser}}", testCase.testCase["_projectId"]);
            testCaseHtml = testCaseHtml.replaceAll("{{.Duration}}", this.help.convertMsToTime(testCase.result.duration));
            // Add run info
            testCaseHtml = testCaseHtml.replaceAll("{{.InfoFilename}}", this.help.getShortFilePath(testCase.testCase["_requireFile"], this.execOptions.testFolder));
            testCaseHtml = testCaseHtml.replaceAll("{{.InfoBrowser}}", testCase.testCase["_projectId"]);
            testCaseHtml = testCaseHtml.replaceAll("{{.InfoStartTime}}", testCase.result.startTime.toUTCString());
            testCaseHtml = testCaseHtml.replaceAll("{{.InfoTotalExecutionTime}}", this.help.convertMsToTime(testCase.result.duration));
            if (showRetries === true) {
                testCaseHtml = testCaseHtml.replaceAll("{{.InfoRetry}}", '<tr><th class="odhin-text">Retry</th><td class="text-secondary-emphasis fst-italic">' +
                    testCase.result.retry.toString() +
                    "</td></tr>");
            }
            else {
                testCaseHtml = testCaseHtml.replaceAll("{{.InfoRetry}}", "");
            }
            // Add errors
            let errors = "";
            let errorFile = "";
            if (testCase.result.status === "failed" ||
                testCase.result.status === "timedOut" ||
                testCase.result.status === "interrupted") {
                if (testCase.result.errors !== undefined) {
                    testCase.result.errors.forEach((err) => {
                        if (err.stack !== undefined && err.stack !== "") {
                            errors += err.stack + "\n\n";
                            let lastLine = err.stack.match(/(.+)([^\s])/g) || "";
                            errorFile = lastLine[lastLine.length - 1].split(" at ")[1];
                        }
                        else {
                            errors += err.message + "\n\n";
                        }
                    });
                }
                testCaseHtml = testCaseHtml.replaceAll("{{.ErrorsTabButton}}", this.help.printErrorsTabButton(testCaseId));
                testCaseHtml = testCaseHtml.replaceAll("{{.Errors}}", this.help.printErrors(errors, errorFile.length > 1 ? this.help.printErrorCode(errorFile) : "", testCaseId));
            }
            else {
                testCaseHtml = testCaseHtml.replaceAll("{{.ErrorsTabButton}}", "");
                testCaseHtml = testCaseHtml.replaceAll("{{.Errors}}", "");
            }
            // Add steps
            let steps = "";
            testCase.result.steps.forEach((step, index) => {
                steps += this.help.printStep(testCaseId, index, step, 10);
            });
            testCaseHtml = testCaseHtml.replaceAll("{{.Steps}}", `<div class="row m-0 p-0"><div class="col m-0 p-0">` + steps + `</div></div>`);
            // Add Attachments
            let screenshots = "";
            let screenshotsComparison = "";
            let screenshotExpected;
            let screenshotActual;
            let screenshotDiff;
            let videos = "";
            let trace = "";
            testCase.result.attachments.forEach(async (attachment) => {
                if (attachment.contentType.startsWith("image/")) {
                    if (attachment.name !== "screenshot") {
                        if (attachment.name.includes("-expected.")) {
                            screenshotExpected = attachment;
                        }
                        else if (attachment.name.includes("-actual.")) {
                            screenshotActual = attachment;
                        }
                        else if (attachment.name.includes("-diff.")) {
                            screenshotDiff = attachment;
                        }
                        else {
                            screenshots +=
                                attachment.path !== undefined
                                    ? this.help.printScreenshot(testCaseId, attachment)
                                    : attachment.body !== undefined
                                        ? this.help.printImage(testCaseId, attachment)
                                        : "";
                        }
                    }
                    else {
                        screenshots +=
                            attachment.path !== undefined
                                ? this.help.printScreenshot(testCaseId, attachment)
                                : attachment.body !== undefined
                                    ? this.help.printImage(testCaseId, attachment)
                                    : "";
                    }
                }
                else if (attachment.contentType.startsWith("video/")) {
                    videos += attachment.path !== undefined ? this.help.printVideo(testCaseId, attachment) : "";
                }
                else if (attachment.contentType.startsWith("application/") && attachment.name === "trace") {
                    trace = attachment.path !== undefined ? this.help.printTrace(testCaseId, attachment) : "";
                }
            });
            if (screenshotExpected !== undefined && screenshotActual !== undefined && screenshotDiff !== undefined) {
                screenshotsComparison += this.help.printScreenshotComparison(testCaseId, screenshotExpected, screenshotActual, screenshotDiff);
            }
            testCaseHtml = testCaseHtml.replaceAll("{{.ScreenshotsTabButton}}", screenshotsComparison !== "" || screenshots !== "" ? this.help.printScreenshotsTabButton(testCaseId) : "");
            testCaseHtml = testCaseHtml.replaceAll("{{.Screenshots}}", screenshotsComparison !== "" || screenshots !== ""
                ? this.help.printScreenshots(screenshotsComparison + screenshots, testCaseId)
                : "");
            testCaseHtml = testCaseHtml.replaceAll("{{.VideosTabButton}}", videos !== "" ? this.help.printVideosTabButton(testCaseId) : "");
            testCaseHtml = testCaseHtml.replaceAll("{{.Videos}}", videos !== "" ? this.help.printVideos(videos, testCaseId) : "");
            testCaseHtml = testCaseHtml.replaceAll("{{.TraceTabButton}}", trace !== "" ? this.help.printTraceTabButton(testCaseId) : "");
            testCaseHtml = testCaseHtml.replaceAll("{{.Trace}}", trace);
            let stdout = testCase.result.stdout.toString();
            testCaseHtml = testCaseHtml.replaceAll("{{.StdoutTabButton}}", stdout !== "" ? this.help.printStdoutTabButton(testCaseId) : "");
            testCaseHtml = testCaseHtml.replaceAll("{{.Stdout}}", stdout !== "" ? this.help.printStdout(testCaseId, stdout) : "");
            let stderr = testCase.result.stderr.toString();
            testCaseHtml = testCaseHtml.replaceAll("{{.StderrTabButton}}", stderr !== "" ? this.help.printStderrTabButton(testCaseId) : "");
            testCaseHtml = testCaseHtml.replaceAll("{{.Stderr}}", stderr !== "" ? this.help.printStderr(testCaseId, stderr) : "");
            testCases += testCaseHtml;
        });
        this.content = this.content.replace("{{.TestCases}}", testCases);
        if (showRetries === true) {
            this.content = this.content.replaceAll("{{.RetryColumn}}", "<th>Retry</th>");
        }
        else {
            this.content = this.content.replaceAll("{{.RetryColumn}}", "");
        }
        // Add Information to the base HTML file
        this.base = this.help.updateHtml(this.base, "{{.BootstrapCss}}", this.bootstrapCss, "css", "bootstrap.css");
        this.base = this.help.updateHtml(this.base, "{{.JQueryDataTablesCss}}", this.jqueryDataTablesCss, "css", "jquery-datatable.css");
        this.base = this.help.updateHtml(this.base, "{{.PrismCss}}", this.prismCss, "css", "prism.css");
        this.base = this.help.updateHtml(this.base, "{{.Css}}", this.css, "css", "style.css");
        this.base = this.help.updateHtml(this.base, "{{.ImgComparisonSliderCss}}", this.imgComparisonSliderCss, "css", "img-comparison-slider.css");
        this.base = this.help.updateHtml(this.base, "{{.ImgComparisonSliderJs}}", this.imgComparisonSliderJs, "js", "img-comparison-slider.js");
        this.base = this.help.updateHtml(this.base, "{{.jQuery}}", this.jQuery, "js", "jquery.js");
        this.base = this.help.updateHtml(this.base, "{{.JQueryDataTablesJs}}", this.jqueryDataTablesJs, "js", "jquery-datatable.js");
        this.base = this.help.updateHtml(this.base, "{{.DataTables}}", this.dataTables, "js", "datatable.js");
        this.base = this.help.updateHtml(this.base, "{{.ChartJsLibrary}}", this.chartJsLibrary, "js", "chartjs-library.js");
        this.base = this.help.updateHtml(this.base, "{{.ChartJs}}", this.chartJs, "js", "chartjs.js");
        this.base = this.help.updateHtml(this.base, "{{.Js}}", this.js, "js", "script.js");
        this.base = this.help.updateHtml(this.base, "{{.BootstrapJs}}", this.bootstrapJs, "js", "bootstrap.js");
        this.base = this.help.updateHtml(this.base, "{{.PrismJs}}", this.prismJs, "js", "prism.js");
        this.base = this.base.replaceAll("{{.Title}}", this.execOptions.title);
        this.base = this.base.replace("{{.RunInfo}}", this.runInfo);
        this.base = this.base.replace("{{.ChartStatus}}", this.chartStatus);
        this.base = this.base.replace("{{.ChartFile}}", this.chartFile);
        this.base = this.base.replace("{{.ChartBrowser}}", this.chartBrowser);
        this.base = this.base.replace("{{.SummaryFiles}}", summaryFiles);
        this.base = this.base.replace("{{.SummaryBrowsers}}", summaryBrowsers);
        this.base = this.base.replace("{{.Content}}", this.content);
        // Create the final HTML report file
        this.help.createFile(this.execOptions.outputFolder + "/" + this.INDEX_FILE, this.base);
        if (this.execOptions.consoleLog === true) {
            console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(` `))}`);
            console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(`-------------------------------------`))}`);
        }
        if (this.execOptions.startServer === true) {
            let port = await this.help.getPortFree();
            const execPromise = node_util_1.default.promisify(child_process_1.exec);
            try {
                if (this.execOptions.consoleLog === true) {
                    console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(` `))}`);
                    console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(`Serving HTML report at http://localhost:${port}. Press Ctrl+C to quit.`))}`);
                    console.log(`${(0, picocolors_1.bold)((0, picocolors_1.blue)(` `))}`);
                }
                const cmd = "node " +
                    path_1.default.resolve(__dirname, "") +
                    "/server.mjs " +
                    port +
                    " " +
                    this.execOptions.outputFolder +
                    " " +
                    this.INDEX_FILE;
                const { stdout, stderr } = await execPromise(cmd);
                if (this.execOptions.consoleLog === true) {
                    console.log(stdout);
                }
                if (this.execOptions.consoleError === true) {
                    console.log(stderr);
                }
            }
            catch (error) {
                if (this.execOptions.consoleError === true) {
                    console.log(error);
                }
            }
        }
    }
}
exports.default = OdhinPlaywrightReporter;
